function NationalCalendar() {
/* based on porting from ICU4J*/
/* interfaces to be implemented by every NationalCalendar class */
	/*  returns the cnumber of days in month */
	this.getDaysInMonth = getDaysInMonth;
	this.getNationalYearName = getNationalYearName;
	this.getMonthName = getMonthName;
	/*  from 0 - 6 */
	this.getDayOfWeekName = getDayOfWeekName;
	/*  from 0 - 30 */
	this.getDayOfMonthName = getDayOfMonthName;
	/* for non Hebrew calendar should invariably return false */
	this.isLeapYear = isLeapYear;
	/* YYYY MM DD from National calendar arguments passed, month is from 0-12, returns the array of  Gregorian YYYY MM DD
	* ported from isu4j */
	this.getGregorianDateFromNational = getGregorianDateFromNational;
	/* YYYY MM DD from Gregorian calendar arguments passed, month is from 0-12, returns the array of National  YYYY MM DD
	* ported from isu4j */
	this.getNationalDateFromGregorian = getNationalDateFromGregorian;
	/* provides national title for GUI checkbox  */
	this.getNationalCalendarCheckboxTitle = getNationalCalendarCheckboxTitle;
	/* being passed YYYY MM DD from National calendar arguments,  formates full national date */
	this.formatNationalDate = formatNationalDate;
	/* being passed full national date, returns array of YYYY MM DD from National calendar  */
	this.parseNationalDate = parseNationalDate;

	_MONTH_NAMES = [/*TISHRI*/ "\u05ea\u05e9\u05e8\u05d9",   
	/*HESHVAN */ "\u05d7\u05e9\u05d5\u05df",
	/*KISLEV*/ "\u05db\u05e1\u05dc\u05d5",
	/*TEVET*/ "\u05d8\u05d1\u05ea",
	/*SHEVAT*/ "\u05e9\u05d1\u05d8",
	/*ADAR_1*/ "\u05d0\u05d3\u05e8 \u05d0\'",
	/*ADAR*/ "\u05d0\u05d3\u05e8 \u05d1\'",
	/*NISAN*/ "\u05e0\u05d9\u05e1\u05df",
	/*IYAR*/ "\u05d0\u05d9\u05d9\u05e8",
	/*SIVAN*/ "\u05e1\u05d9\u05d5\u05df",
	/*TAMUZ*/ "\u05ea\u05de\u05d5\u05d6",
	/*AV*/ "\u05d0\u05d1",
	/*ELUL*/ "\u05d0\u05dc\u05d5\u05dc"];

	var _ADAR_NAME = "\u05d0\u05d3\u05e8";  

	_SINGLE_NAMES = ["", 
	"\u05d0",  
	"\u05d1",  
	"\u05d2",  
	"\u05d3",  
	"\u05d4",
	"\u05d5",
	"\u05d6",
	"\u05d7",
	"\u05d8"];

	_TENS_NAMES = ["\u05d9",  
	"\u05db",  
	"\u05dc",  
	"\u05de",  
	"\u05e0",
	"\u05e1",
	"\u05e2",
	"\u05e4",
	"\u05e6"];

	_HUNDREDS_NAMES = ["\u05e7",  
	"\u05e8", 
	"\u05e9",
	"\u05ea",  
	"\u05ea\u05e7",  
	"\u05ea\u05e8",  
	"\u05ea\u05e9",
	"\u05ea\u05ea",
	"\u05ea\u05ea\u05e7"];
		
	var _HOUR_PARTS = 1080;
	var _DAY_PARTS = 24*1080;
   
	var _MONTH_DAYS = 29;
	var _MONTH_FRACT = 12*1080 + 793;
	var _MONTH_PARTS = 29*24*1080 + 12*1080 + 793;
	  
	var BAHARAD = 11*1080 + 204;

	var JAN_1_1_JULIAN_DAY = 1721426;

	var _MONTH_LENGTH_TABLE =  [
		[   30,	    30,	    30	],		 //Tishri
		[   29,	    29,	    30	],		 //Heshvan
		[   29,	    30,	    30	],		 //Kislev
		[   29,	    29,	    29	],		 //Tevet
		[   30,	    30,	    30	],		 //Shevat
		[   30,	    30,	    30	],		 //Adar I
		[   29,	    29,	    29	],		 //Adar II
		[   30,	    30,	    30	],		 //Nisan
		[   29,	    29,	    29	],		 //Iyar
		[   30,	    30,	    30	],		 //Sivan
		[   29,	    29,	    29	],		 //Tammuz
		[   30,	    30,	    30	],		 //Av
		[   29,	    29,	    29	]		 //Elul
	];

	var _MONTH_START_TABLE =  [
		[    0,		0,		0  ],
		[   30,	    30,	    30  ],		// Tishri
		[   59,	    59,	    60  ],		// Heshvan
		[   88,	    89,	    90  ],		// Kislev
		[  117,	   118,	   119  ],		// Tevet
		[  147,	   148,	   149  ],		// Shevat
		[  147,	   148,	   149  ],		// Adar I
		[  176,	   177,	   178  ],		// Adar II
		[  206,	   207,	   208  ],		// Nisan
		[  235,	   236,	   237  ],		// Iyar
		[  265,	   266,	   267  ],		// Sivan
		[  294,	   295,	   296  ],		// Tammuz
		[  324,	   325,	   326  ],		// Av
		[  353,	   354,	   355  ]		// Elul
	];

	var _LEAP_MONTH_START_TABLE =  [
		[    0,		0,		0  ],
		[   30,	    30,	    30  ],		// Tishri
		[   59,	    59,	    60  ],		// Heshvan
		[   88,	    89,	    90  ],		// Kislev
		[  117,	   118,	   119  ],		// Tevet
		[  147,	   148,	   149  ],		// Shevat
		[  177,	   178,	   179  ],		// Adar I
		[  206,	   207,	   208  ],		// Adar II
		[  236,	   237,	   238  ],		// Nisan
		[  265,	   266,	   267  ],		// Iyar
		[  295,	   296,	   297  ],		// Sivan
		[  324,	   325,	   326  ],		// Tammuz
		[  354,	   355,	   356  ],		// Av
		[  383,	   384,	   385  ]		// Elul
	];
	
	var _GREGORIAN_MONTH_COUNT_TABLE =  [
		[  31,  31,   0,   0 ], // Jan
		[  28,  29,  31,  31 ], // Feb
		[  31,  31,  59,  60 ], // Mar
		[  30,  30,  90,  91 ], // Apr
		[  31,  31, 120, 121 ], // May
		[  30,  30, 151, 152 ], // Jun
		[  31,  31, 181, 182 ], // Jul
		[  31,  31, 212, 213 ], // Aug
		[  30,  30, 243, 244 ], // Sep
		[  31,  31, 273, 274 ], // Oct
		[  30,  30, 304, 305 ], // Nov
		[  31,  31, 334, 335 ] // Dec
	];

	function getDaysInMonth(month, year){
		if(!this.isLeapYear(year) && (month >= 5))  
			month++;

		switch(month){
			case 1: //HESHVAN
			case 2: //KISLEV
				return _MONTH_LENGTH_TABLE[month][_yearType(year)];
			default:
				return _MONTH_LENGTH_TABLE[month][0];
		}	
	}

	function getMonthName(month, year){
		if(!isLeapYear(year) && (month >= 5)) {  
			if(month == 5)
				return _ADAR_NAME;

			month++;
		}

		return _MONTH_NAMES[month];	
	}

	function _makeFixesInTwoDigitDate(str){
		str = str.replace("\u05d9\"\u05d5"  ,
				"\u05d8\"\u05d6");
		str = str.replace("\u05d9\"\u05d4"  ,
				"\u05d8\"\u05d5");
		if(str.charAt(str.length - 1) == "\"")
			str = str.replace("\""  ,"\'");

		return str;	
	}

	function _getTwoDigitDate(date){
		if(date >= 10) {
			var tens = _TENS_NAMES[ parseInt(date/10,10) - 1];
			date = date % 10;
			return ( tens + "\"" + _SINGLE_NAMES[date] );
		}
		return ( _SINGLE_NAMES[date]  +   "\'" );	
	}

	function getNationalYearName(year){
		year = year % 1000;
		var hundreds = parseInt(year / 100,10);
		var twoDigitDate = _getTwoDigitDate(year%100);
		return ( _HUNDREDS_NAMES[hundreds - 1] + _makeFixesInTwoDigitDate(twoDigitDate) );
	
	}

	function getDayOfWeekName(day){
		if(day == 6)
			return "\u05e9\u05d1\u05ea";
		else
			return ( _SINGLE_NAMES [day + 1] + "\'");	
	}

	function getDayOfMonthName(day){
		var twoDigitDate = _getTwoDigitDate(day);
		return (  _makeFixesInTwoDigitDate(twoDigitDate) );	
	}

	function formatNationalDate(year, month, day){
		return ( getDayOfMonthName(day) + " " + getMonthName(month, year) + " " + getNationalYearName(year) );	 
	}

	function parseNationalDate(str){
		var year = 5000;
		var month = -1;
		var tmp0 = 0;
		var tmp2 = 0;
		var index;

		for (var i = 0;i < _MONTH_NAMES.length;i++) {
			index = str.indexOf(_MONTH_NAMES[i]);
			if(index != -1) {
				month = i;
				part0 = str.substring(0,index);
				part2 = str.substring(index + _MONTH_NAMES[i].length);
				break;
			}
		}

		if((month == -1)) {  
			index = str.indexOf(_ADAR_NAME);
			if(index != -1) {
				month = 5;
				part0 = str.substring(0,index);
				part2 = str.substring(index + _ADAR_NAME.length);
			} else {
				return new Array(5770, 1, 1);
			}
		}


		for (var i = 8;i >= 0;i--) {
			if(part0.indexOf(_SINGLE_NAMES[i + 1]) != -1) {
				tmp0 += i + 1;
			}
			if(part0.indexOf(_TENS_NAMES[i]) != -1) {
				tmp0 += (i + 1) * 10;
			}

			if(part2.indexOf(_SINGLE_NAMES[i + 1])!= -1) {
				tmp2 += i + 1;
			}
			if(part2.indexOf(_TENS_NAMES[i]) != -1) {
				tmp2 += (i + 1)  * 10;
			}
		}

		for (var i = 8;i >= 0;i--) {
			if(part0.indexOf(_HUNDREDS_NAMES[i]) != -1) {
				tmp0 += (i + 1)  * 100;
				part0 = part0.substring(_HUNDREDS_NAMES[i].length + 1);
			}

			if(part2.indexOf(_HUNDREDS_NAMES[i]) != -1) {
				tmp2 += (i + 1)  * 100;
				part2 = part2.substring(_HUNDREDS_NAMES[i].length + 1);
			}
		}

		if(tmp2 > tmp0) {
			year += tmp2; day = tmp0;
		} else {
			year += tmp0; day = tmp2;
		} 

		if(!isLeapYear(year) && (month > 5))  
			month--;

		return new Array(year, month, day);	
	}

	function _yearType(year){
		var yearLength = _getYearLength(Number(year));
		if(yearLength > 380){
			yearLength -= 30;
		}

		switch(yearLength){
			case 353:
				return 0;
			case 354:
				return 1;
			case 355:
				return 2;
		}
	}

	function _getYearLength(year){
		return _getStartOfYear(year+1) - _getStartOfYear(year);
	}

	function _getStartOfYear(year){				            
		var months = Math.floor((235 * year - 234) / 19);

		var fractions = months * _MONTH_FRACT + BAHARAD;
		var day  = months * 29 + Math.floor(fractions / _DAY_PARTS);
		fractions = fractions % _DAY_PARTS;

		var weekDay = day % 7;

		if(weekDay == 2 || weekDay == 4 || weekDay == 6){
			day += 1;
			weekDay = day % 7;
		}

		if(weekDay == 1 && fractions > 15*_HOUR_PARTS+204 && !isLeapYear(year) )
			day += 2;
		else if(weekDay == 0 && fractions > 21*_HOUR_PARTS+589 && isLeapYear(year-1) )
			day += 1;           
				
		return day;
	}

	 function isLeapYear(year){	
		var x = (year*12 + 17) % 19;
		return x >= ((x < 0) ? -7 : 12);
	}
	
	 function getNationalDateFromGregorian(year, month, date){
		var julianDay = _getJulianDayFromGregorian(year, month, date);
		var tmpDay = julianDay - 347997;
		var tmpMonth = Math.floor((tmpDay * _DAY_PARTS) / _MONTH_PARTS);
		var year = Math.floor((19 * tmpMonth + 234) / 235) + 1;
		var yearStart  = _getStartOfYear(year);
		var dayOfYear = (tmpDay - yearStart);

		while (dayOfYear < 1){
			year--;
			yearStart  = _getStartOfYear(year);
			dayOfYear = (tmpDay - yearStart);
		}

		var typeofYear = _yearType(year);
		var monthStart = isLeapYear(year) ? _LEAP_MONTH_START_TABLE : _MONTH_START_TABLE;
		var month = 0;
		while(dayOfYear > monthStart[month][typeofYear]){
				month++;
		}
		month--;

		var dayOfMonth = dayOfYear - monthStart[month][typeofYear];

		if(!isLeapYear(year)  &&  month >= 5)  
			month--;

		return new Array(year, month, dayOfMonth);
	}

	 function getGregorianDateFromNational (year, month, date){			
		if(!isLeapYear(year)  &&  month>=5)
			month++;

		if(month >12 || (!isLeapYear(year) && month > 11)){
			alert("incorrect National date");			
		}

		var tmpYear = year;
		var tmpMonth =  month;
		var tmpDate =  date;
		var day = _getStartOfYear(tmpYear);
		if(tmpMonth != 0){
			if(isLeapYear(tmpYear)){
				day += _LEAP_MONTH_START_TABLE[tmpMonth][_yearType(tmpYear)];
			} else{
				day += _MONTH_START_TABLE[tmpMonth][_yearType(tmpYear)];
			}
		}

		var julianDay =  (tmpDate + day + 347997);
		var gregorianEpochDay = julianDay - JAN_1_1_JULIAN_DAY;
		var remainder = new Array(1);
		var nCycle400 = _fDivision(gregorianEpochDay , 146097, remainder);

		var nCycle100 = _fDivision(remainder[0] , 36524, remainder);

		var nCycle4 = _fDivision(remainder[0] , 1461, remainder);

		var nCycle1 = _fDivision(remainder[0] , 365, remainder);
		var year = 400*nCycle400 + 100*nCycle100 + 4*nCycle4 + nCycle1;
		var dayOfYear = remainder[0];
		if(nCycle100 == 4 || nCycle1 == 4){
			dayOfYear = 365;
		}else{
			++year;
		}

		var isLeap = (year%4 == 0) &&
				(year%100 != 0 || year%400 == 0);
		var correction = 0;
		var march1 = isLeap ? 60 : 59; // zero-based DOY for March 1
		if(dayOfYear >= march1){ correction = isLeap ? 1 : 2; }
		var month = Math.floor((12 * (dayOfYear + correction) + 6) / 367);
		var dayOfMonth = dayOfYear - _GREGORIAN_MONTH_COUNT_TABLE[month][isLeap?3:2] + 1;

		return new Array(year, month, dayOfMonth);
	}

	function getNationalCalendarCheckboxTitle() {
		return "\u05dc\u05d5\u05d7 \u05e9\u05e0\u05d4 \u05e2\u05d1\u05e8\u05d9";
	}

	function _fDivision(numerator, denominator, remainder){
	
		if(numerator >= 0){
			remainder[0] = (numerator % denominator);
			return Math.floor(numerator / denominator);
		}
		var quotient = Math.floor(numerator / denominator);
		remainder[0] = numerator - (quotient * denominator);
		return quotient;
	}

	function _getJulianDayFromGregorian(year, month, date){
		var isLeapYear = (year%4 == 0) && ((year%100 != 0) || (year%400 == 0));
		year = year - 1;
		var julianDay = 365*year + Math.floor(year/4) - Math.floor(year/100) +
			Math.floor(year/400) + JAN_1_1_JULIAN_DAY - 1;

		if(month != 0){
			julianDay += _GREGORIAN_MONTH_COUNT_TABLE[month][isLeapYear?3:2];
		}
		
		julianDay += date;
		return julianDay;
	}  
}		
