
var Months = [          /*Muharram*/	"\u0645\u062d\u0631\u0645", 
						/*Safar*/    	"\u0635\u0641\u0631", 
						/*Rabi I*/   	"\u0631\u0628\u064a\u0639 \u0623\u0648\u0644", 
						/*Rabi II*/  	"\u0631\u0628\u064a\u0639 \u062b\u0627\u0646\u064a", 
						/*Jumada I*/ 	"\u062c\u0645\u0627\u062f \u0623\u0648\u0644", 
						/*Jumada II*/	"\u062c\u0645\u0627\u062f \u062b\u0627\u0646\u064a", 
						/*Rajab*/		"\u0631\u062c\u0628", 
						/*Shaban*/		"\u0634\u0639\u0628\u0627\u0646", 
						/*Ramadan*/		"\u0631\u0645\u0636\u0627\u0646", 
						/*Shawwal*/		"\u0634\u0648\u0627\u0644", 
						/*Dhul-Qidah*/	"\u0630\u0648 \u0627\u0644\u0642\u0639\u062f\u0629", 
						/*Dhul-Hijjah*/	"\u0630\u0648 \u0627\u0644\u062d\u062c\u0629"
						];

var WeekDays = [         /*Sun*/	"\u0627\u0644\u0623\u062d\u062f", 
						 /*Mon*/	"\u0627\u0644\u0625\u062b\u0646\u064a\u0646", 
						 /*Tue*/	"\u0627\u0644\u062b\u0644\u0627\u062b\u0627\u0621", 
						 /*Wed*/	"\u0627\u0644\u0623\u0631\u0628\u0639\u0627\u0621", 
						 /*Thu*/	"\u0627\u0644\u062e\u0645\u064a\u0633", 
						 /*Fri*/	"\u0627\u0644\u062c\u0645\u0639\u0629", 
						 /*Sat*/	"\u0627\u0644\u0633\u0628\u062a"
						 ];


function NationalCalendar() {

    /* interfaces to be implemented by every NationalCalendar class */
    /*  returns the number of days in month */
    this.getDaysInMonth = getDaysInMonth;

    this.getNationalYearName = getNationalYearName;

    this.getMonthName = getMonthName;
    /*  from 0 - 6 */
    this.getDayOfWeekName = getDayOfWeekName;
    /*  from 0 - 30 */
    this.getDayOfMonthName = getDayOfMonthName;

    this.isLeapYear = civilLeapYear;
    /* YYYY MM DD from National calendar arguments passed, month is from 0-12, returns the array of  Gregorian YYYY MM DD
    * ported from icu4j */
    this.getGregorianDateFromNational = islamicToGregorian;
    /* YYYY MM DD from Gregorian calendar arguments passed, month is from 0-12, returns the array of National  YYYY MM DD
    * ported from icu4j */
    this.getNationalDateFromGregorian = gregorianToIslamic;
    /* provides national title for GUI checkbox  */
    this.getNationalCalendarCheckboxTitle = getNationalCalendarCheckboxTitle;
    /* being passed YYYY MM DD from National calendar arguments,  formates full national date */
    this.formatNationalDate = formatNationalDate;
    /* being passed full national date, returns array of YYYY MM DD from National calendar  */
    this.parseNationalDate = parseDate;

}



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                           //
//                                  Islamic National Calendar public APIs                                                    //
//                                                                                                                           //
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*getDaysInMonth*/
function getDaysInMonth(month, year) {
    // summary: returns the number of days in the month used
    var length = 0;
    length = 29 + ((month + 1) % 2);
    if ((month == 11) && civilLeapYear(year))
        length++;

    return length;
}


/*getNationalYearName*/
function getNationalYearName(year) {
    return (year);
}
/*getMonthName*/
function getMonthName(month) {
    return Months[month];
}
/*getDayOfWeekName*/
function getDayOfWeekName(day) {
    return WeekDays[day];
}
/*getDayOfMonthName*/
function getDayOfMonthName(day) { return day; }

/*formatNationalDate*/
function formatNationalDate(year, month, day) {
    return (day + " " + Months[month] + " " + year);
}


/*isLeabYear*/
function civilLeapYear(year) {
    return (14 + 11 * year) % 30 < 11;

}

/*getGregorianDateFromNational*/

function islamicToGregorian(year, month, date) {

    var jd = islamic_to_jd(year, month + 1, date);

    return (jd_to_gregorian(jd));
}

/*getNationalDateFromGregorian*/

function gregorianToIslamic(year, month, date) {

    var jd = gregorian_to_jd(year, month + 1, date)

    var year, month, day;

    jd = Math.floor(jd) + 0.5;
    year = Math.floor(((30 * (jd - ISLAMIC_EPOCH)) + 10646) / 10631);
    month = Math.min(12,
                Math.ceil((jd - (29 + islamic_to_jd(year, 1, 1))) / 29.5) + 1);
    day = (jd - islamic_to_jd(year, month, 1)) + 1;

    return new Array(year, month - 1, day);
}


//////////////////////////////////////////

/*
*		the parser function called by dateObj.parse()
*/


function parseDate(dateObject) {
    var sDate = dateObject.toString();
    	var year = 1400;
		var month = -1;
		var index;

		for (var i = 0;i < Months.length;i++) {
			index = dateObject.indexOf(Months[i]);
			if(index != -1) {
				month = i;
				day = dateObject.substring(0,index);
				year = dateObject.substring(index + Months[i].length);
				break;
			}
		}
		return new Array(year, month, day);	
};

function getNationalCalendarCheckboxTitle() {
		return "\u0627\u0644\u062a\u0642\u0648\u064a\u0645 \u0627\u0644\u0647\u062c\u0631\u064a";
	}


//////////////////////////////////////////

/*
*		Internal methods used for calculations
*/

function getIndex(arr, str) {

    for (i = 0; i < arr.length; i++) {
        if (arr[i] == str)
            return i;
    }
    return -1;
};

function yearStart(year) {

    return (year - 1) * 354 + Math.floor((3 + 11 * year) / 30.0);

}

function monthStart(year, month) {

    return Math.ceil(29.5 * month)
                    + (year - 1) * 354 + Math.floor((3 + 11 * year) / 30.0);
}







function mod(a, b) {
    return a - (b * Math.floor(a / b));
}


function leap_gregorian(year) {
    return ((year % 4) == 0) &&
            (!(((year % 100) == 0) && ((year % 400) != 0)));
}



var GREGORIAN_EPOCH = 1721425.5;

function gregorian_to_jd(year, month, day) {
    return (GREGORIAN_EPOCH - 1) +
           (365 * (year - 1)) +
           Math.floor((year - 1) / 4) +
           (-Math.floor((year - 1) / 100)) +
           Math.floor((year - 1) / 400) +
           Math.floor((((367 * month) - 362) / 12) +
           ((month <= 2) ? 0 :
                               (leap_gregorian(year) ? -1 : -2)
           ) +
           day);
}


function jd_to_gregorian(jd) {
    var wjd, depoch, quadricent, dqc, cent, dcent, quad, dquad,
        yindex, dyindex, year, yearday, leapadj;

    wjd = Math.floor(jd - 0.5) + 0.5;
    depoch = wjd - GREGORIAN_EPOCH;
    quadricent = Math.floor(depoch / 146097);
    dqc = mod(depoch, 146097);
    cent = Math.floor(dqc / 36524);
    dcent = mod(dqc, 36524);
    quad = Math.floor(dcent / 1461);
    dquad = mod(dcent, 1461);
    yindex = Math.floor(dquad / 365);
    year = (quadricent * 400) + (cent * 100) + (quad * 4) + yindex;
    if (!((cent == 4) || (yindex == 4))) {
        year++;
    }
    yearday = wjd - gregorian_to_jd(year, 1, 1);
    leapadj = ((wjd < gregorian_to_jd(year, 3, 1)) ? 0
                                                  :
                  (leap_gregorian(year) ? 1 : 2)
              );
    month = Math.floor((((yearday + leapadj) * 12) + 373) / 367);
    day = (wjd - gregorian_to_jd(year, month, 1)) + 1;

    return new Array(year, month - 1, day);
}

var ISLAMIC_EPOCH = 1948439.5;

function islamic_to_jd(year, month, day) {
    return (day +
            Math.ceil(29.5 * (month - 1)) +
            (year - 1) * 354 +
            Math.floor((3 + (11 * year)) / 30) +
            ISLAMIC_EPOCH) - 1;
}


var GREGORIAN_EPOCH = 1721425.5;

function gregorian_to_jd(year, month, day) {
    return (GREGORIAN_EPOCH - 1) +
           (365 * (year - 1)) +
           Math.floor((year - 1) / 4) +
           (-Math.floor((year - 1) / 100)) +
           Math.floor((year - 1) / 400) +
           Math.floor((((367 * month) - 362) / 12) +
           ((month <= 2) ? 0 :
                               (leap_gregorian(year) ? -1 : -2)
           ) +
           day);
}





