/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Presentation Services
*
* (C) Copyright IBM Corp. 2007, 2008  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} ***
*******************************************************************************/

//Inheritance
function extend(base, extension){
   for ( var property in base ){
      try{
         extension[property] = base[property];
         //todo if extension has property - not change it.
      }
      catch( warning ){}
   }
}
// Represents all WTable components on the page.
var RegisteredTables = {
 list : new Array(),
 
getWTable : function getWTable(id){
	var table = null;
	for(var i=0;i<this.list.length;++i){
		if(this.list[i] && this.list[i].getId()==id){
			table = this.list[i];
			break;
		}
	}
	if(table){
		debugInfo("get WTABLE  " + table.toString());
	}else{
		table = window["registeredTable_" + id];
		debugInfo("no such a WTable object");
	}
	return table;
},
 
registerWTable : function(obj){
	var table = this.getWTable(obj._id);
	if(table){
		var i = indexOfImpl(this.list, table);
		this.list.splice(i,1,obj);
	}else{
	this.list.push(obj);
	}
	window["registeredTable_" + obj._id] = obj;
	debugInfo("WTable  registered " + obj.getId());
},
updateCache : function(id, param){
	var table = this.getWTable(id);	
	if(table){	
		table.updateCache(param);
	}	
},
onDataRequest : function(id){
	var table = this.getWTable(id);	
	if(table){	
		table._dataFetcher.onDataRequest();
	}	
},
doUserAction : function( id, func){
	var table = this.getWTable(id);
	
	if(table){
		debugInfo("WTable " + id + "doUserAction performed ");
		table.doUserAction(func);
	}	
},
onDragDrop : function(id){
	var table = this.getWTable(id);
	if(table){
		debugInfo("WTable " + id + "drag drop performed ");
		table.onDragDrop();
	}	
},

select : function(id, nb){
	var table = this.getWTable(id);
	if(table){
		debugInfo("WTable " + id + "select request " + nb);
		table.select(nb);
	}	
},

changeAllSelected: function(id){
	var table = this.getWTable(id);
	if(table){
		debugInfo("WTable " + id + "change all selected request ");
		table.changeAllSelected();
	}	
},


exportData: function(id){
	var table = this.getWTable(id);
	if(table){
		debugInfo("WTable " + id + "export data request ");
		table.exportData();
	}
},

rowRefresh : function(id, nb){
	var table = this.getWTable(id);
	if(table){
		debugInfo("WTable " + id + "rowRefresh request " + nb);
		table.rowRefresh(nb);
	}	
},

nextTablePage : function(id){	
	var table = this.getWTable(id);
	if(table){
		debugInfo("WTable " + id + "next page request");
		table.pageNext();
	}
},

prevTablePage : function(id){
	var table = this.getWTable(id);
	if(table){
		debugInfo("WTable " + id + "prev page request");
		table.pagePrev();
	}
},
page : function(id, textentry){
	var table = this.getWTable(id);
	if(table){
		debugInfo("WTable " + id + "page request");
		table._pageTextEntry = textentry;
		table.page();
	}
	return false;
},

changeRowsPerPage : function(id, rowsPerPage) {
	var table = this.getWTable(id);
	if(table){
		table._pageSize = rowsPerPage;
		table.changeRowsPerPage(rowsPerPage);
	}
	
}
}

WTable = function(id, formName, actionName, actionValue, actionNameEnc, wclhidden, wclanchor){
	extend(new Observer(), this);
	extend(new Observable(), this);
	//table id
	this._id = '';
	//body id
	this._bodyId = '';
	//page size - number of rows per page
	this._pageSize = 10;
	// cache Size - number of pages in cache
	this._cacheSize = 25; 
	this._cachedAhead = 10;
	this._cachedBackward = 10;
	// current Page number
	this._currentPage = 0;
	this._oldCurrentPage = 0;
	this._nextPage = 0;
	this._pageTextEntry = undefined;
	this._pageGoButton = undefined;
	
	this._offset =0;
	
	//first visible row
	this._firstVisibleRow = 0;
	//last visible row
	this._lastVisibleRow = 0;
	//rows count
	this._rowCount =0;
	this._rowsPerView = 10;
	this._numPages = 0;
		
	this._visibleRows = new Array();
	this._data = new Array();
	this.cachedPages = new ArrayList();
	
	// inline action bar
	this._actionBar = new Array();

	//Observer	
	this.notify = this.mynotify;
	
	// init
	if ( arguments.length ) this.init( id, formName, actionName, actionValue, actionNameEnc, wclhidden, wclanchor );
	
	return this;
}
/**
* Prototype functions
*/
WTable.prototype = {
	/**
   * Psudo-Constructor that concrete classes call in their conctructor.
   * @param {string} formName
   * @param {string} wclhidden
   * @param {string} wclanchor
   **/
	init: function( id, formName, actionName, actionValue, actionNameEnc, wclhidden, wclanchor ){
		if ( !formName ) {
         alert( "Fatal Error: WTable init called without formName specified.");
         return;
         }
        this._id = id;
        this._dataFetcher = new DataFetcher(formName,wclhidden, wclanchor);
        //enable it 
        this.registerAction(actionName, actionValue, actionNameEnc);

		RegisteredTables.registerWTable(this);
	},
	initPagingFeature : function( bodyId, cachedAhead, cacheSize, pageSize, rowCount, currentPage, rowsPerView, pageTextEntry, pageGoButton ){
		this._bodyId = bodyId;
		this._cachedAhead = cachedAhead; 
		this._cacheSize = cacheSize;
		this._pageSize = pageSize;
		this._rowCount = rowCount;
		this._pageTextEntry = pageTextEntry;
		this._pageGoButton = pageGoButton;
		this._currentPage = Number(currentPage);
		this._rowsPerView = rowsPerView;
		this.initCacheParameters();
		debugInfo("WTable " + this._id + " Paging feature inited");
		this.updatePageValues();
//		var exec = new Executor(this, 500, 'updateCache', null);
//		Executor.asyncExecute(exec);
		this.updateCache();
	},
	/**
	 * always have in cache the first and the last page
	 * always have stored in cache current page with all possible sorting combinations
	 * always have cacheAhead pages from currentPage - maximum is cacheSize - 4
	 * minimum number of cachedBackward is one 
	 */
	initCacheParameters : function(){
		if(this._cachedAhead > this._cacheSize - 4){
			if (this._cacheSize - 4 < 0)
				this._cachedAhead = 0;
			else
				this._cachedAhead = this._cacheSize - 4;
			this._cachedBackward = 1; 
			return;
		}
		if(this._cachedAhead < 0){
			this._cachedAhead = 0;
		}
		if(this._cachedBackward < 0){
			this._cachedBackward = 0;
		}
		if(this._cachedAhead*2 < this._cacheSize -3){
			this._cachedBackward = this._cachedAhead;
		}else{
			this._cachedBackward = (this._cacheSize-3) - this._cachedAhead;
		}	
	},
	updatePageValues : function( ){
		//console.debug("updatePageValues"+ this._pageSize);
		var rowsOnLastPage = this._rowCount % this._pageSize;
		debugInfo("rowsOnLastPage: "+ rowsOnLastPage);
		this._numPages = ((this._rowCount - rowsOnLastPage) / this._pageSize) + ((rowsOnLastPage > 0) ? 1 : 0);
		debugInfo("_numPages: "+ this._numPages);
		if (this._currentPage < 0){
      		this._currentPage = 0;
      	}
    	if (this._currentPage > 0 && this._currentPage >= this._numPages){
      		this._currentPage = this._numPages - 1;
      	}
      	this._firstVisibleRow = this._pageSize * this._currentPage;
      	this._lastVisibleRow = Number(this._firstVisibleRow) + Number(((this._rowCount - this._firstVisibleRow) > rowsOnLastPage) ? this._pageSize : rowsOnLastPage);
    	//console.debug("WTable " + this._id + " Page values updated fvr= " + this._firstVisibleRow + " lvr= " + this._lastVisibleRow);
	},
	pageChanged : function( ){
		debugInfo("WTable " + this._id + " Page changed");
		this.updatePageValues();
		var updateparam = null;
		var interval = 1000;
		// if collapse table is set then it is unnecessery to prefetch any pages and update view
		if (document.getElementById(this._bodyId) == null) {
			this.updateInfo();
		} else if(this.isCached(this._firstVisibleRow, this._lastVisibleRow)){
			this.updateInfo();
			this.updateView();
			setupTableAjax(this._bodyId);
			this.updateCache(updateparam);
			//this.addServerSync();
		}else{
			updateparam = "pageChanged";
			interval = 1;
			this.updateCache(updateparam);
		} 
	},
	
	rowsPerPageChanged : function( rowsPerPage ){
	
		this.updatePageValues();
		var updateparam = null;
		var interval = 1000;
		// if collapse table is set then it is unnecessery to prefetch any pages and update view
		if (document.getElementById(this._bodyId) == null) {
			this.updateInfo();
		} else if(this.isCached(this._firstVisibleRow, this._lastVisibleRow)){
			this.updateInfo();
			this.updateView();			
			setupTableAjax(this._bodyId);
			
			//call updateCache in the callback of addServerSyncRows and introduce some delay for successive row controller clicks...
			var exec = new Executor(this, 1000, 'addServerSyncRows',rowsPerPage);
 			Executor.asyncExecute(exec);
			//this.addServerSyncRows(rowsPerPage);
			//this.updateCache(updateparam);
			//console.debug("here before add server sync for row controller...");
		}else{
			updateparam = "changeRowsPerPage("+rowsPerPage+")";
			interval = 1;
			this.updateCache(updateparam);
		} 
	},
	addServerSync : function(){
		this._dataFetcher.addServerSyncRequest("page(" + this._currentPage + ")");
	},
	
	addServerSyncRows : function(rowsPerPage){
		var query = "updateServer("+rowsPerPage+")" ;
		var dr = new DataRequest(-1,-1,0, query)
		this._dataFetcher.addDataRequest(dr);
		//this._dataFetcher.addServerSyncRequest("updateServer("+rowsPerPage+")");
		
	},
	
	getRowsPerPage : function(pageNb){
		var ret = {start:0,end:0};	
		ret.start = pageNb*this._pageSize;
		if(pageNb == this._numPages-1){
			var rowsOnLastPage = this._rowCount % this._pageSize;
			ret.end=Number(ret.start) + Number(((this._rowCount - ret.start) > rowsOnLastPage) ? this._pageSize : rowsOnLastPage);
		}else{
			ret.end=Number(ret.start) + Number(this._pageSize);
		}
		debugInfo("WTable " + this._id + " Rows per " + pageNb + " start= " + ret.start + " end= " + ret.end );
		return ret;
	},
	removePage : function(pageNb){
		var p = this.getRowsPerPage(pageNb);
		for(var i = p.start; i<p.end;++i){
			this._data[i]= null;
		}
		debugInfo("WTable " + this._id + " Page " + pageNb + " removed from cache");
		return (this.isPageCached(pageNb));
	},	
	isPageCached : function(pageNb){
		var p = this.getRowsPerPage(pageNb);
		debugInfo("WTable " + this._id + " Page " + pageNb + " is cached");
		return (this.isCached(p.start,p.end))
	},	
	canRemove :  function( index ){
		var c = this._currentPage;
		if(index == c 
		|| (index < c + this._cachedAhead && index > c )
		|| (index > c - this._cachedBackward && index < c)){
			return false;
		}
		return true;
	},
	getCachedNextPages : function(){
		var list = new Array();
		var c = this._currentPage;
		for(var i = 0; i < this._cachedAhead ; i++){
			c = this.getNextPage(c);
			list.push(c);
		}
		return list;
	},
	getCachedPrevPages : function(){
		var list = new Array();
		var c = this._currentPage;
		for(var i = 0; i < this._cachedBackward ; i++){
			c = this.getPrevPage(c);
			list.push(c);
		}
		return list;
	},
	getNextPage : function(pageNb){
		var next = 0;
		if(pageNb == this._numPages-1) next = 0;
		else next = pageNb + 1;
		debugInfo("WTable " + this._id + " Next page for page " + pageNb + " is " + next);
		return next;
	},
	getPrevPage : function(pageNb){
		var prev = 0;
		if(pageNb == 0) prev =  (this._numPages-1);
		else prev = pageNb - 1;
		debugInfo("WTable " + this._id + " Prev page for page " + pageNb + " is " + prev);
		return prev;
	},
	
	updateCache : function( param ){
		if( param != null){
			if(!this.isPageCached(this._currentPage)){
				p = this.getRowsPerPage(this._currentPage);
				debugInfo("p.start: "+ p.start);
				debugInfo("p.end: "+ p.end);
				debugInfo("param: "+ param);
				var dr = new DataRequest(p.start, p.end-p.start,0, param);
				dr.add(p.start, p.end);
				if(this._dataFetcher.indexOf(dr) == -1){
					this._dataFetcher.addDataRequest(dr);
					//alert("not in queue");				
				}				
			}else{
				this.pageChanged();
			}
			return;
		}
		
		param = "dataRequest";
		
		// first time - load _cacheAhead number of pages from currentPage
		if(this.isEmpty()){
			debugInfo("WTable " + this._id + " Cache is empty");
			var count = (Number(this._cachedAhead)+1)*this._pageSize;
			var last = Number(this._firstVisibleRow) + count;
			if(last > this._rowCount){
				count = this._rowCount - this._firstVisibleRow;
				last = this._rowCount;
			}
			
			var dr = new DataRequest(this._firstVisibleRow,count,0,param);
			dr.add(this._firstVisibleRow, last);
			this._dataFetcher.addDataRequest(dr);
			return;
		}
		var next = this._currentPage;
		var dr1 = new DataRequest(-1,-1, 0,param);
		var x = null;
		for(var i=0;i<this._cachedAhead; i++){			
			if(!this.isPageCached(next)){
				p = this.getRowsPerPage(next);
				
				dr1.add(p.start, p.end);
				//var dr = new DataRequest(p.start, p.end-p.start,0, param);
				//this._dataFetcher.addDataRequest(dr);
			}
			next = this.getNextPage(next);
		}
		this._dataFetcher.addDataRequest(dr1);
		
		
		
	},
	getInterval : function(start){
		start = Number(start);
		if(start>= this._data.length || start < 0){
			return null;
		}
		var tstart = undefined;
		var tend = undefined;
		var cont = true;
		var i = start;
		while(cont){
			if(this._data[i]){
				if(tstart == undefined){
					tstart = i;
				}		
				tend = i; 
			}else{
				if(tstart != undefined){
				cont = false;
				tend = i;
				}
			}
			i++;
		}
		return {start : tstart, end : tend};
	},
	
	refreshCache : function(){
		var dr = new DataRequest(-1,-1,0,"refreshCache");
		var interval = this.getInterval(0); 
		var start = 0;
		while(interval != null){
			dr.add(interval.start, interval.end);
			start = Number(interval.end)+1;
			interval = this.getInterval(start);
		}
		this._dataFetcher.addDataRequest(dr);
	},
	
	controlCacheSize : function(oldSize){
		debugInfo("WTable " + this._id + " control cache size");
		var tempcachedPages = new Array();
		var pages = new Array();
		for( var i = 0; i < this._numPages; ++i ){
			if(this.isPageCached(i) && this.cachedPages.indexOf(i)== -1){
				tempcachedPages.push(i);
			}
		}
		var newSize =  Number(this.cachedPages.size()) + Number(tempcachedPages.length);
		while(tempcachedPages.length > 0) {
			this.cachedPages.push(tempcachedPages.shift());
		}
		if(newSize <= Number(this._cacheSize)) return;
		var diff = newSize - Number(this._cacheSize);
		for(var i = 0 ; i< this.cachedPages.size(); i++){
			if(diff==0) break;
			var page = this.cachedPages.get(i);
			if(this.canRemove(page)){
			this.removePage(page);
			this.cachedPages.removeFrom(i);
			diff--;
		}
		}	
	},
	
	onNewRow : function(index){
		var maxRows = this._pageSize * (this._cacheSize);
		var datacount = 0;
		for(var i=0; i< this._data.length; ++i){
			if(this._data[i]){
				datacount++;
			}
		}
		if(datacount > maxRows){
			debugInfo("WTable " + this._id + " onNewRow ");
			var old = this.cachedPages.length;
			this.controlCacheSize(old);
		}
	},
	
	
	
	registerAction : function( actionName, actionValue, actionNameEnc ){
		if(actionValue == "dataRequest"){
			this._dataFetcher.enable( actionValue, actionName, actionNameEnc );
			this._dataFetcher.addObserver(this);
		}
	},
	
	isCached : function(first, last){
		for(var i=first; i< last; ++i){
			if(!this._data[i]){
				return false;
			}
		}
		return true;
	},
	isEmpty : function(){
		debugInfo("WTable " + this._id + " Is empty" + this._data.length);
		return (this._data.length == 0)
	},
	
	
	toString: function( ){
		return "WTable with id: " + this._id;
	},
	
	getId: function( ){
		return this._id;
	},
	
	/**
	*implements Observer
	*/
	mynotify : function(ev){
		var event = ev.callback._event;
		if(event == "dataRequest" || event == "refreshCache" || event.indexOf("prepareData") != -1){
			debugInfo("notify in WTable - dataRequest");
			this.onDataRequestCompleted(ev.request);
		}
		
		if(event == "pageChanged"){
			//console.debug("pageChanged event... ");
			this.onDataRequestCompleted(ev.request);
			debugInfo("notify in WTable - pageChanged");
			if(this.isPageCached(this._currentPage)){
				// it shouldn't happen when doing move prev or move next, most often after go {pageNb} action			
				if(this._oldCurrentPage == this.getPrevPage(this._currentPage)){
					//alert("unexpected problem while moving next...");
				}
				//this.onDataRequestCompleted(ev.request);
				this.pageChanged();
			}
		}
		if(event.indexOf("select") != -1){
			// it was select request
			this.onSelectRequestCompleted(ev.request);
			if((event.indexOf("all") != -1 || event.indexOf("none") != -1)){
				this.refreshCache();
			}
		}
		if(event.indexOf("rowRefresh") != -1){
			// it was rowRefresh request
			this.onRowRefreshRequestCompleted(ev.request);
		}
		if(event.indexOf("onDragDrop") != -1){
			this.refreshCache();
		}
		if(event.indexOf("exportData") != -1){
			this.onExportDataRequestCompleted(ev.request);
		}
		
		if(event.indexOf("changeRowsPerPage") != -1){
			this.onRowsPerPageDataRequestCompleted(ev.request);
		}
		
		if(event.indexOf("updateServer") != -1) {
			//do nothing
			//console.debug("event updateServer.... ");
			this.updateCache(null);
		}
		
	},
	
	onRowsPerPageDataRequestCompleted: function(req) {
		var tempContent = document.createElement("div");
          tempContent.setAttribute("id","ServerDataResponse");
          debugInfo("Reading in response");
          tempContent.innerHTML = req.responseText;
          debugInfo("Done reading in response");
          var tbodies = tempContent.getElementsByTagName("TBODY");
          var doAgain = true;
          var tbody = null;
          for (var i = 0; i < tbodies.length && doAgain == true; i++) {
          // If we find an AJAX container in the response
           debugInfo("tbodies: "+tbodies[i].getAttribute("id")+ "_bodyId"+ this._bodyId );
         	if (tbodies[i].getAttribute("id") == this._bodyId ) {
          		tbody = tbodies[i];
          		doAgain = false;
          	}
          }
          if(tbody){
          
           //IE8 - fix - Ajax update doesn't work
      		var noOfRows = 0;
      		if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
      			noOfRows = tbody.childNodes.length;
      		}else{
      			noOfRows = tbody.rows.length;
      		}     
          this._pageSize = noOfRows;
          	for (var i=0;i<noOfRows;++i) {
          			if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
          				var tr = tbody.childNodes[i];
          			}else{
          				var tr = tbody.rows[i];
          			}
          			var index = Number(tr.getAttribute("row"));
          			this._data[index] = tr;
          			
          		}  
         	this.pageChanged();
         	
         	var anyDivs = tempContent.getElementsByTagName("DIV");
			var pageControllerDiv = null;
			var divId = null;
			for ( var i = 0; i < anyDivs.length ; i++) {
				if (anyDivs[i].getAttribute("id") && (anyDivs[i].getAttribute("id").indexOf("TPC") != -1)){
					pageControllerDiv = anyDivs[i];
					divId = anyDivs[i].getAttribute("id") ;
					break;
				}
				
				
			}
			if(pageControllerDiv !=null){
				var docPageContDiv = document.getElementById(divId);
				docPageContDiv.innerHTML = pageControllerDiv.innerHTML;
			}
					
          			
          	 var removeDiv = document.getElementById("ServerDataResponse");
          if(removeDiv){
          	document.removeChild(removeDiv);        
          } 
          	
          }
		
	},
	
	
	onExportDataRequestCompleted : function(req){
//		alert(req.responseText);
		handleExportData(req.responseText);
	},
	onSelectRequestCompleted : function(req){
		debugInfo("cacheData after select request");
		var tempContent = document.createElement("div");
          tempContent.setAttribute("id","ServerDataResponse");
          debug("Reading in response");
          tempContent.innerHTML = req.responseText;
          debug("Done reading in response");
          var tbodies = tempContent.getElementsByTagName("TBODY");
          var doAgain = true;
          var tbody = null;
          for (var i = 0; i < tbodies.length && doAgain == true; i++) {
          // If we find an AJAX container in the response
         	if (tbodies[i].getAttribute("id") == this._bodyId ) {
          		tbody = tbodies[i];
          		doAgain = false;
          	}
          }
          if(tbody){
          	 //IE8 - fix - Ajax update doesn't work
      		var noOfRows = 0;
      		if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
      			noOfRows = tbody.childNodes.length;
      		}else{
      			noOfRows = tbody.rows.length;
      		}    
          	for (var i=0;i<noOfRows;++i) {
          			if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
          				var tr = tbody.childNodes[i];
          			}else{
          				var tr = tbody.rows[i];
          			}
          			var index = Number(tr.getAttribute("row"));
          			this._data[index] = tr;
          			this.onNewRow(index);         		
          			this.updateSelectView(index);
          		}    	
          }
	},	
	
	onRowRefreshRequestCompleted : function(req){
		debugInfo("cacheData after select request");
		var tempContent = document.createElement("div");
          tempContent.setAttribute("id","ServerDataResponse");
          debug("Reading in response");
          tempContent.innerHTML = req.responseText;
          debug("Done reading in response");
          var tbodies = tempContent.getElementsByTagName("TBODY");
          var doAgain = true;
          var tbody = null;
          for (var i = 0; i < tbodies.length && doAgain == true; i++) {
          // If we find an AJAX container in the response
         	if (tbodies[i].getAttribute("id") == this._bodyId ) {
          		tbody = tbodies[i];
          		doAgain = false;
          	}
          }
          if(tbody){
          	//IE8 - fix - Ajax update doesn't work
      		var noOfRows = 0;
      		if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
      			noOfRows = tbody.childNodes.length;
      		}else{
      			noOfRows = tbody.rows.length;
      		}  
          	for (var i=0;i<noOfRows;++i) {
          			if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
          				var tr = tbody.childNodes[i];
          			}else{
          				var tr = tbody.rows[i];
          			}
          			var index = Number(tr.getAttribute("row"));
          			this._data[index] = tr;
          			this.onNewRow(index);         		
          			this.updateSelectView(index);
          		}    	
          }
	},	
	
	onDataRequestCompleted : function(req){
		this.cacheData(req);
		
	},
	cacheData : function(req){
	
		  debugInfo("cacheData");
		  var tempContent = document.createElement("div");
          tempContent.setAttribute("id","ServerDataResponse");
          debug("Reading in response");
          tempContent.innerHTML = req.responseText;
          debug("Done reading in response");
          var divs = tempContent.getElementsByTagName("DIV");
          var doAgain = true;
          var div = null;
          for (var i = 0; i < divs.length && doAgain == true; i++) {
         // If we find an AJAX container in the response
         	if (divs[i].getAttribute("id") == "DATA_CONTAINER") {
          		div = divs[i];
          		doAgain = false;
          	} 
          }
          if(div){
          
          	var parentNode = div.parentNode;
          	if(parentNode.id == this._id){
          		var tbody = div.firstChild.firstChild;
          		//TODO valueChanged
          		var ps = tbody.getAttribute("rpp");
          		if(this._pageSize != Number(ps)){
          			this._pageSize = Number(ps);
          		}
          		ps = tbody.getAttribute("mpc");
          		if(this._cacheSize != Number(ps)){
          			this._cacheSize = Number(ps);
          		}
          		ps = tbody.getAttribute("pcc");
          		if(this._cachedAhead != Number(ps)){
          			this._cachedAhead = Number(ps);
          		} 
          		
          		 //IE8 - fix - Ajax update doesn't work
          		var noOfRows = 0;
          		if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
          			noOfRows = tbody.childNodes.length;
          		}else{
          			noOfRows = tbody.rows.length;
          		}     
          		for (var i=0;i<noOfRows;++i) {
          			if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
          				var tr = tbody.childNodes[i];
          			}else{
          				var tr = tbody.rows[i];
          			}
          			var index = Number(tr.getAttribute("row"));
          			this._data[index] = tr;
          			this.onNewRow(index);         		
          		}    	
          		if (tbody.nextSibling != null) { // this is in case inline action bar show
          			if (tbody.nextSibling.nextSibling != null) {
          				var inlineBody = tbody.nextSibling.nextSibling;
          				var noOfRows = 0;
		          		if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
		          			noOfRows = inlineBody.childNodes.length;
		          		}else{
		          			noOfRows = inlineBody.rows.length;
		          		}  
          				for (var i=0;i<noOfRows;++i) {
		          			if((WClient.isBrowserInternetExplorer()) && (WClient.major == 8)){
		          				var tr = inlineBody.childNodes[i];
		          			}else{
		          				var tr = inlineBody.rows[i];
		          			}
		          			var index = Number(tr.getAttribute("row"));
		          			if (this._actionBar != null && i == 0 && index >= this._currentPage*this._pageSize+2) {
		          				this._actionBar[index] = tbody.nextSibling; 
		          			}
		          			this._data[index] = tr;
		          			this.onNewRow(index);         		
		          		}
          			}
          		} 
          	}
          	else{
          		alert("it is not this table data")
           // it is not this table data
		           
          	}
         }
          var removeDiv = document.getElementById("ServerDataResponse");
          if(removeDiv){
          	document.removeChild(removeDiv);        
          } 
	},	
	updateInfo : function( ){ 
		 var id = this._id.slice(3);
		 var tpcid = "TPC_" + this._id.slice(3);
		 var tpcDiv = document.getElementById(tpcid);
	  	 var text = tpcDiv.innerHTML;
	  	 var a = text.split(" ");
	  	 var cp = false;
	  	 var success = false;
	  	 for(var b= 0; b<a.length; ++b){
	  	 	if(Number(a[b]) > 0 && !cp){
	  	 		a[b] = this._currentPage+1;
	  	 		cp = true;
	  	 	}
	  	 	if(cp && (b == a.length-1) ){
	  	 		a[b] = this._numPages;
	  	 		success = true;
	  	 		break;
	  	 	}	 	
	  	 }
	  	 if(success){
	  	 	text = "";
	  	 	for(var b= 0; b<a.length; ++b){
	  	 		text += a[b];
	  	 		if(b != a.length-1){
	  	 			text += " ";
	  	 		}
	  	 	}
	  	 	tpcDiv.innerHTML = text;	 	
	  	 }
	  	 var te = document.getElementById(this._pageTextEntry);
	  	 if(te){
	  	 	te.value = Number(this._currentPage) + 1;
	  	 }
	  	 
	  	 var pageBtn  = document.getElementById(this._pageGoButton);
	  	 if(pageBtn && (this._numPages == 1)){
	  	 	pageBtn.setAttribute("class","b3");
	  	 	pageBtn.setAttribute("disabled", "disabled");
	  	 	pageBtn.setAttribute("style","background-color: #ddd;");
	  	 }else{
	  	 	pageBtn.setAttribute("class","b1");
	  	 	pageBtn.removeAttribute("disabled");
	  	 	pageBtn.setAttribute("style","");
	  	 }
	  	 
	  	 var div = document.getElementById("PREV_"+id);
	  	 if(this._currentPage > 0 && div){
	  	 	//show prev button 	
	  	 	div.firstChild.style.display ='none';
	  	 	div.lastChild.style.display ='inline';
	  	 }else if(div){
	  	 	div.firstChild.style.display ='inline';
	  	 	div.lastChild.style.display ='none';
	  	 }
	  	 div = document.getElementById("NEXT_"+id);
	  	 if(this._currentPage +1 != this._numPages && div){
	  	 	//show prev button 	
	  	 	div.firstChild.style.display ='none';
	  	 	div.lastChild.style.display ='inline';
	  	 }else if(div){
	  	 	div.firstChild.style.display ='inline';
	  	 	div.lastChild.style.display ='none';
	  	 }
	  	 
	  	 debugInfo("WTable " + this._id + " Info updated");
	},
	updateView : function( ){
	 	 this._visibleRows = new Array();
		 for(var i = this._firstVisibleRow; i < this._lastVisibleRow; ++i){
			 this._visibleRows.push(this._data[i]);
		 }
	  	 
	  	var tbody = document.getElementById(this._bodyId);
		 var scrollBarVisible = false;
		 if((this._lastVisibleRow - this._firstVisibleRow )> this._rowsPerView){
		 	scrollBarVisible = true;
		 }else{
		 	scrollBarVisible = false;
		 }
		 this.updateScrollPosition(tbody,scrollBarVisible);
		   	 
	  	 if (tbody.nextSibling != null) { // this is in case inline action bar show
	  	 	tbody.parentNode.removeChild(tbody.nextSibling.nextSibling);
	  	 	tbody.parentNode.removeChild(tbody.nextSibling);
	  	 }
	  	 //replace old rows with the new ones.
	  	 if(tbody){ 
		  	 var mrows = tbody.getElementsByTagName("TR");
		  	 while(tbody.hasChildNodes()){
		  	 	tbody.removeChild(tbody.firstChild);
		  	 }
		  	 var c = 0;
		  	 var inlineBody = document.createElement("tbody");
		  	 var inline = false;
		  	 
		  	 //IE8 - fix - Ajax update doesn't work
		  	 var visibleRowsFragment = document.createDocumentFragment();
			 var actionBarFragment = document.createDocumentFragment();			
		  	 for(var i = 0; i < this._visibleRows.length; ++i){
		  	 	if(this._visibleRows[i]){
		  	 		var index = Number(this._visibleRows[i].getAttribute("row"));
		  	 		if (this._actionBar[index] == null) {
		  	 			visibleRowsFragment.appendChild(this._visibleRows[i]);
						c++;
		  	 		} else {
		  	 			actionBarFragment.appendChild(this._actionBar[index]);
		  	 			tbody.parentNode.appendChild(inlineBody);	 
		  	 			inline = true;
		  	 			break;
		  	 		}
		  	 	}
		  	 }
		  	 tbody.appendChild(visibleRowsFragment.cloneNode(true));
			 tbody.parentNode.appendChild(actionBarFragment.cloneNode(true));
		  	 		
		  	 if (inline) {
		  	 	 inlineBody.appendChild(visibleRowsFragment.cloneNode(true));
			  	 inlineBody.setAttribute("dr", this._visibleRows.length);
		  	 }
		  	 
		  	tbody.setAttribute("rpv", this._rowsPerView);
		  	tbody.setAttribute("dr", c);
		  	debugInfo("WTable " + this._id + " View updated");
	  	 }
	  	
	},
	
	updateScrollPosition : function(tbody, scrollBarVisible) {
		var scrollDiv = document.getElementById(tbody.getAttribute("SPID"));
		if(scrollBarVisible){
			if ( WClient.isBrowserInternetExplorer() ){
				scrollDiv.style.overflowY = 'scroll';
				scrollDiv.onscroll = "setScrollPositionAjax('"+ tbody.getAttribute("SPID") + "');";
			}else{
				scrollDiv.setAttribute("style","overflow-y:scroll;overflow:-moz-scrollbars-vertical;");
				scrollDiv.setAttribute("onscroll", "setScrollPositionAjax('"+ tbody.getAttribute("SPID") + "');");
			}
						
		}
		else if(scrollDiv && !scrollBarVisible){
			if ( WClient.isBrowserInternetExplorer() ){
				scrollDiv.style.overflowY = 'hidden';
			}else{
				scrollDiv.setAttribute("style","");
			}
			
		}
			
	},
	
	updateSelectView : function(index) {
		var tbody = document.getElementById(this._bodyId);
		if(tbody){ 
		  	 var mrows = tbody.getElementsByTagName("TR");
		  	 var tr = this._data[index].cloneNode(true);
		  	 for (var i=0; i<mrows.length; i++)
		  	 	if (mrows[i].id == tr.id) {
		  	 		tbody.replaceChild(tr, mrows[i]);
		  	 		break;
		  	 	}
		}
	},
	
	onDragDrop : function(){
		var dr = new DataRequest(-1,-1,0,"onDragDrop");
		this._dataFetcher.addDataRequest(dr);
		//this.refreshCache();
	},
	
	select : function(nb){
		var query = "select("+ nb + ")";
		var dr = new DataRequest(-1,-1,0, query)
		this._dataFetcher.addDataRequest(dr);
	},
	
	changeAllSelected : function(){
		var query = "changeSelected()";
		var dr = new DataRequest(-1,-1,0, query)
		this._dataFetcher.addDataRequest(dr);
	},
	
	exportData : function(){
		var query = "exportData()";
		var dr = new DataRequest(-1,-1,0, query)
		this._dataFetcher.addDataRequest(dr);
	},
	
	changeRowsPerPage: function(rowsPerPage) {
		//var query = "changeRowsPerPage("+rowsPerPage+")";
		//var dr = new DataRequest(-1,-1,0, query)
		//this._dataFetcher.addDataRequest(dr);
		this._pageSize = rowsPerPage;
		this.rowsPerPageChanged(rowsPerPage);
		
	},
	

	rowRefresh : function(nb){
		var query = "rowRefresh("+ nb + ")";
		var dr = new DataRequest(-1,-1,0, query)
		this._dataFetcher.addDataRequest(dr);
	},
	
	/**
	 * Go to the next page of table data
	 */	
	pageNext : function(   ){
		this.page( this._currentPage + 1 );
	},
	
	/**
	 * Go to the previous page of table data
	 */	
	pagePrev : function( ){
		this.page( this._currentPage - 1 );
	},
	
	/**
	* Go to specific page
	* @param {int} pageNb page number
	*/
	page : function( pageNb ){
		if(arguments.length){
		this.updatePageValues();
		if( pageNb < 0) pageNb = 0;
		if( pageNb >= this._numPages )  pageNb = this._numPages -1;
		if( pageNb == this._currentPage){
			//do nothing
			return;
		}
		debugInfo("WTable " + this._id + " Page(" + pageNb + ")");
		this._oldCurrentPage = this._currentPage;
		this._currentPage = pageNb;
		
		this.pageChanged();
		}else{
			//todo if undefined try to find...
			var te = document.getElementById(this._pageTextEntry);
			var nb = Number(te.value)-1;
			this.page(nb);
		}
	}
}

DataFetcher = function(formName,wclhidden, wclanchor) {
	//inheritance
	extend(new Observer(), this);
	extend(new Observable(), this);
	
	this._formName ='';
	this._wclhidden ='';
	this._wclanchor ='';
	
	this._actualRequest=undefined;
	this._requests = new Array();
	this._serverSyncRequests = new Array();
	this._actServerSyncRequest = undefined;
	
	// init
	if ( arguments.length ) this.init( formName, wclhidden, wclanchor );
	return this;
}

DataFetcher.prototype = {
	init : function( formName,wclhidden, wclanchor, id ){
		this._id  = id;
		this._formName = formName;
        this._wclhidden = wclhidden;
        this._wclanchor = wclanchor;
        this._enabled = false;
        this._actionName = '';
		this._actionValue = '';
		this._actionNameEnc = '';	
		
		//this.watch("_actualRequest", this.onDataRequest);
        
	},
	
	indexOf : function(dr){
		for(var i = 0; i< this._requests.length; i++){
			var adr = this._requests[i];
			if(dr.equals(adr)){
				return i;
			}
		}
		return -1;
	},
	
	addDataRequest : function(dr) {
		if(dr._rows.length == 0 && dr._event.indexOf("dataRequest") != -1){
			return;
		}
		debugInfo("WTable.DataFetcher new data request(" + dr._start + "," + dr._count + "," + dr._event +")");
		if(this._actualRequest != undefined){
		if(this._actualRequest.equals(dr)){
			return;
		}}
		
		this._requests.push(dr);
		var exec = new Executor(this, 200, 'onDataRequest');
  		Executor.asyncExecute(exec);
	},
	
	addServerSyncRequest : function(value){
		debugInfo("WTable.DataFetcher new serverSync request("+value+")");
		if(this._actServerSyncRequest != undefined){
		if(this._actServerSyncRequest == value){
			return;}}
		this._serverSyncRequests.push(value);
		this.onDataRequest();
	},
	
	onDataRequest : function() {
		if(!this.isWaiting()){
			if(this._serverSyncRequests.length > 0){
				this._actServerSyncRequest = this._serverSyncRequests.shift();
				debugInfo("WTable.DataFetcher " + this._actServerSyncRequest.toString());
				this.updatePageRequest(this._actServerSyncRequest);
			}else if(this._requests.length > 0){
			this._actualRequest = this._requests.shift();
			if(this._actualRequest){
			debugInfo("WTable.DataFetcher " + this._actualRequest.toString());
			debugInfo("WTable.DataFetcher new DataRequest start" );
			if(this._actualRequest._event == "pageChanged"){
				debugInfo("WTable.DataFetcher " + this._actualRequest.toString());
			}
			if (this._actualRequest._event.toString().indexOf("changeSelected") == 0 ) {
				// it is Change All Selected request
				this.request(this._actualRequest._event);
			} else if(this._actualRequest._event.toString().indexOf("select") != -1){
				// it is select request...
				this.request(this._actualRequest._event);
			} else if(this._actualRequest._event.toString().indexOf("rowRefresh") != -1){
				// it is rowRefresh request...
				this.request(this._actualRequest._event);
			} else if(this._actualRequest._event.toString().indexOf("exportData") != -1){
				// it is exportData request...				
				this.request(this._actualRequest._event);				
			}else if(this._actualRequest._event.toString().indexOf("onDragDrop") != -1){
				this.request(null);
			} else if (this._actualRequest._event.toString().indexOf("prepareData") != -1) {
				this.request(this._actualRequest._event);
			} else if (this._actualRequest._event.toString().indexOf("prepareHeaders") != -1) {
				this.request(this._actualRequest._event);
			}
			else if(this._actualRequest._event.toString().indexOf("changeRowsPerPage") != -1){
				// it is RowController request...				
				this.request(this._actualRequest._event);				
			}
			else if(this._actualRequest._event.toString().indexOf("updateServer") != -1){ 
				this.request(this._actualRequest._event);	
			}else{
				this.fetchData(this._actualRequest);
			}
			}
			}
		}
	},
	
	request : function(action){
		 if (document != null && document.forms != null && this._formName != null) {
      	var form = document.forms[this._formName];
      	
      	if (form != null) {
         	if (this._actionName != null && action != null) {
            	eval("form." + this._actionNameEnc + ".value = '" + action + "'");
            	eval("form." + this._wclhidden + ".value = '" + this._actionName + "'");
         	}
         	debug("actionNameEnc = " + this._actionNameEnc);
         	debug("actionValue = " + this._actionValue);

         	debug("AJAX request with action = " + form.action);
         	if (action.toString().indexOf("rowRefresh") != -1) {
	         	startDataRequest(form, action, this.onStateChange, this);
         	} else {
         		startDataRequest(form, null, this.onStateChange, this);
         	}
         	// Reset the form values.
         	eval("form." + this._actionNameEnc + ".value = ''");
         	eval("form." + this._wclhidden + ".value = ''");
      	} }
	},
	
	fetchData : function(dr) {
		var actionValue = this._actionValue + "(";
		if(dr._rows.length > 0){
			for(var i=0; i< dr._rows.length; i++){
			actionValue +=  dr._rows[i].from;
			actionValue += "-"
			actionValue += dr._rows[i].to;
			if(i <  dr._rows.length -1){
				actionValue += ",";
			}
			}
			actionValue += ")"
		}else{
			var from = dr._start;
			var offset = dr._count;
	  		debugInfo("WTable.DataFetcher fetchdata (" + from +"," + offset + ")");
	  		actionValue = this._actionValue + "(" + from + ","+ offset +")";
	  }
	  if (document != null && document.forms != null && this._formName != null) {
      	var form = document.forms[this._formName];
      	
      	if (form != null) {
         	if (this._actionName != null) {
            	eval("form." + this._actionNameEnc + ".value = '" + actionValue + "'");
            	eval("form." + this._wclhidden + ".value = '" + this._actionName + "'");
         	}

         	debug("actionNameEnc = " + this._actionNameEnc);
         	debug("actionValue = " + this._actionValue);

         	debug("AJAX request with action = " + form.action);
         	startDataRequest(form, null, this.onStateChange, this);
         	// Reset the form values.
         	eval("form." + this._actionNameEnc + ".value = ''");
         	eval("form." + this._wclhidden + ".value = ''");
      }
   	}
	},
	
	updatePageRequest : function(value){
		//console.debug("WTable.DataFetcher updatePageRequest (" + value +")");
	  var actionValue = value;
	  if (document != null && document.forms != null && this._formName != null) {
      	var form = document.forms[this._formName];
      	
      	if (form != null) {
         	if (this._actionName != null) {
            	eval("form." + this._actionNameEnc + ".value = '" + actionValue + "'");
            	eval("form." + this._wclhidden + ".value = '" + this._actionName + "'");
         	}

         	debug("actionNameEnc = " + this._actionNameEnc);
         	debug("actionValue = " + this.actionValue);

         	debug("AJAX request with action = " + form.action);
         	startServerSyncRequest(form, null, this.onServerSyncResponse(), this);
         	// Reset the form values.
         	eval("form." + this._actionNameEnc + ".value = ''");
         	eval("form." + this._wclhidden + ".value = ''");
      }
	  }
	},
	
	isWaiting : function(){
		return (this._actualRequest != undefined && this._actServerSyncRequest != undefined);
	},
	
	enable : function(actionValue, actionName, actionNameEnc){
		this._actionName = actionName;
        this._actionValue = actionValue;
        this._actionNameEnc = actionNameEnc;
        this._enabled = true;
	
	},
	
	disable : function(){
		this._enabled = false;
	},
	
	onComplete : function( req ){
		debugInfo("onComplete");
		
		//we can notify what kind of requets it was
		this.notifyObservers( { callback: this._actualRequest , request: req } );
		this._actualRequest = undefined;
		var exec = new Executor(this, 400, 'onDataRequest');
 		Executor.asyncExecute(exec);
		return false;
	},
	
	onServerSyncComplete : function(){
		this._actServerSyncRequest = undefined;
		this.onDataRequest();
	},
	
	onStateChange : function(response, ioArgs){
			debugInfo("data response ");
			var req = ioArgs.xhr;
			if(requestor != undefined 
				&& requestor._actualRequest != undefined 
				&& requestor._actualRequest._event != undefined){
				if(requestor._actualRequest._event.indexOf("select") != -1 || 
				   requestor._actualRequest._event.indexOf("onDragDrop") != -1){
					handleStateChange(response, ioArgs);
				}
			}
			
			if(requestor != undefined) {
				requestor.onComplete(req);			
			}
			isAJAXOutstanding = false;					
	},
	
	
	
	onServerSyncResponse : function(response, ioArgs){		
			debugInfo("data response ");
			isAJAXOutstanding = false;
			requestor.onServerSyncComplete();
	}
	

}

DataRequest = function(offset, count, priority, event){
	this._start = 0;
    this._count = 0;
    this._rows = new Array();
    this._priority = 0;
    this._event = undefined;

    // init
	if ( arguments.length ) this.init( offset, count, priority, event );
	return this;
}
DataRequest.prototype = {
	init : function( offset,count, priority, event ){
		this._start = offset;
        this._count = count;
        this._priority = priority;
        this._event = event;
	},

	equals: function(datareq){
		if(datareq._event == "onDragDrop"){
			return false;
		}
		for(var i=0;i<this._rows.length; i++){
			
		}
		return (this._offset == datareq._offset && this._count == datareq._count && this._event == datareq._event );
	},
	add : function(f, t){
		var found = false;
		for(var i =0; i< this._rows.length; i++){
			var r = this._rows[i];
			if(r.to == f){
				r.to = t;
				found = true;
				break;
			}
		}
		if(!found){
			this._rows.push({from : f, to: t });
		}
		
	},
	compareTo: function(datareq){
		if(this.equals(datareq)) return 0;
		if(this._offset < datareq._offset && this._count+this._offset <= datareq._offset) return -1;
		if(this._offset >= datareq._offset + datareq._count ) return -1;
		if(this._offset >= datareq._offset && this._offset + this._count <= datareq._offset + datareq._count ) return 1; 
	},
	
	toString : function(){
		return this._event;
	}	
}	

AjaxUpdater = function( ) {

	// init
	if ( arguments.length ) this.init( arguments );
	return this;
}

AjaxUpdater.prototype = {
	init : function( args ){
	},
	update :function( ){
		
	}
}

// Observable
Observable = function( ){
	this._observers = new Array();
	// init
	
	return this;
}
Observable.prototype = {
	
	addObserver : function(o){
		debugInfo("addObserver");
		this._observers.push(o);
	},
	removeObserver : function( o ){
		debugInfo("removeObserver");
		for ( var i = 0; i < this._observers.length; i++ ){
    		if ( this._observers[i] == o ) this._observers[i] = undefined;
  		}
	},
	notifyObservers : function( ev ){
		for ( var i = 0; i < this._observers.length; i++ ){
    		if ( this._observers[i] ){
    			debugInfo("notify observer");
      			this._observers[i].notify( ev );
    		}
  		}
	}
}

// Observer
Observer = function( ){
	// init
	return this;
}
Observer.prototype = {
	notify : function( ev ){
		return;
	}
}

function Executor(obj, delay, method, param){
  this._id = 'exc_' + (Executor.counter++);
  this._object = obj;
  this._delay = Number(delay);
  this._timerId = 0;
  this._method = method;
  this._arg = param;
  Executor.pending[this._id] = this;
}
Executor.counter = 0;
Executor.pending = {};

Executor.prototype.execute = function(){
  this._object[this._method](this._arg);                                       
  delete Executor.pending[this._id];
};

Executor.asyncExecute = function (exc){
	Executor.pending[exc._id]._timerId = setTimeout('if (Executor.pending["' + exc._id + '"]) Executor.pending["' + exc._id + '"].execute()', exc._delay);
};


function indexOfImpl(list, obj) {
	if (!Array.prototype.indexOf)
	{
	  Array.prototype.indexOf = function(obj /*, from*/)
	  {
	    var len = this.length;
	
	    var from = Number(arguments[1]) || 0;
	    from = (from < 0)
	         ? Math.ceil(from)
	         : Math.floor(from);
	    if (from < 0)
	      from += len;
	
	    for (; from < len; from++)
	    {
	      if (from in this &&
	          this[from] == obj)
	        return from;
	    }
	    return -1;
	  };
	} else {
		return list.indexOf(obj);
	}
}

// TG
function getRowNumber(element){
	var parent = element;
	while(parent != null && (parent.tagName != 'TR' || parent.getAttribute('row') == null)){
		parent = parent.parentNode;
	}

	if(parent == null){	// element wasn't in table
		return -1;
	}else{
		return parent.getAttribute('row');
	}
}

function isInSelectedRow(element){
	var parent = element;
	while(parent != null && (parent.tagName != 'TD' || !(parent.className == 'tbl2' || parent.className == 'tbls2' || parent.className == 'tbl4' || parent.className == 'tbls4'))){
		parent = parent.parentNode;
	}

	if(parent == null){	// element wasn't in table
		return false;
	}else{
		return (parent.className == 'tbls2' || parent.className == 'tbls4');
	}
}

function assosArrSize(arr) {
	var l = arr.length ? --arr.length : -1;
	for (var k in arr) {
		l++;
	}
	return l;
}