/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Presentation Services
*
* (C) Copyright IBM Corp. 2002, 2008  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} ***
*******************************************************************************/

/*******************************************************************************
* This file contains all of the javascript functions used by the WCL renderers.
* The functions defined here must be stateless -- that is, no member variables
* can be defined, so all state information must be passed into the function
* as parameters.
*******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// frame reload manager functions
////////////////////////////////////////////////////////////////////////////////

/** FrameReloadManager, AWFrame **
 * framePath - the path from the current page to the frame to be reloaded
 * reloadURL - the url to load into that frame
 */
function fMgr( framePath, reloadURL )
{
   if (framePath != null && reloadURL != null) {
      var frame = eval(framePath);
      if (frame != null && frame.location != null) {
         var useReload = false;
         try {
            // We have to check if the reloadURL matches the end of the
            // frame's location.href (except for the anchor, if any).
            // If so, this is the case that Netscape may ignore on
            // location.replace() so we set a flag to use location.reload(true)
            // instead.  This is done inside a try/catch block because
            // examining the location.hash and location.href may fail
            // of the location is a URL on anther domain.
            var hashLength = 0;
            if (frame.location.hash != null)
               hashLength = frame.location.hash.length;
            if (hashLength > 0) {
               var offset = frame.location.href.length - (hashLength + reloadURL.length);
               if ((offset >= 0) && (frame.location.href.indexOf(reloadURL) == offset)) {
                  useReload = true;
               }
            }
         }
         catch(e) { }
         if (useReload)
            frame.location.reload(true);
         else
            frame.location.replace(reloadURL);
      }
   }
}

////////////////////////////////////////////////////////////////////////////////
// form input component functions
////////////////////////////////////////////////////////////////////////////////


/** WComboBox **
 * textfield - the editable text field for the combobox
 * value - the value of the selected option
 * addOption - the value of the "add" option
 * comboImg - the image rendered next to the label of the dropdown
 * textImg - the image rendered next to the text field
 * statusName - the style class id for the status (normal, required, error)
 */
function editbx(textfield, value, addOption, comboImg, textImg, statusName) {
   if (textfield != null) {
      if (value != addOption) {
      	 if (textfield.parentNode.className == "te8") {
      	 	textfield.parentNode.className = "";
      	 }
         textfield.disabled = true;
         textfield.className = "te7";
         if (comboImg != null) {
         	if (comboImg.style != null) { //@auiml - check for null to avoid javascript errors
            	comboImg.style.display = "inline";
	        }
         }
         if (textImg != null) {
         	if (textImg.style != null) { //@auiml - check for null to avoid javascript errors
            	textImg.style.display = "none";
            }
         }
      } else {
         textfield.disabled = false;
         if (statusName != null) {
         	if (statusName == "te8" || statusName == "te4") {
         		if (textfield.parentNode.tagName == "DIV") {
         			textfield.parentNode.className = statusName;
         			textfield.className = "te2";
         		} else {
	         		var div = document.createElement("DIV");
	         		div.className = statusName;
	         		textfield.className = "te2";
	         		var parent = textfield.parentNode;
	         		div.appendChild(textfield);
	         		parent.appendChild(div);
         		}
         	} else {
	            textfield.className = statusName;
         	}
         }
         if (comboImg != null) {
         	if (comboImg.style != null) {  //@auiml - check for null to avoid javascript errors
            	comboImg.style.display = "none";
            }
         }
         if (textImg != null) {
         	if (textImg.style != null && !(statusName == "te8" || statusName == "te4")) {  //@auiml - check for null to avoid javascript errors
            	textImg.style.display = "inline";
            }
         }
      }
   }
}


/** WComboBox and WSelectionBox to cause an onChange in Netscape 7 with up and down arrow keys **
 * selObj - the selection object
 * event - the key event
 */
function chgEvt(selObj, event) {

   if(WClient.isBrowserMozilla() && WClient.isBrowserVersion7Up())
   {
   	  //@auiml - need to check for alt key
      if (!(event.altKey))
      {
	      var wEvent = new WEvent(event);
	      if(wEvent.getKeyCode() == 38 || wEvent.getKeyCode() == 40)
    	  {
        	 selObj.blur();
	         selObj.focus();
    	  }
      }
   }

}



////////////////////////////////////////////////////////////////////////////////
// complex component functions
////////////////////////////////////////////////////////////////////////////////

/** WMessageBox **
 * name - the name of the close hyperlink
 * ids - the ids associated with the close hyperlink
 * formName - the encoded name of the form
 * wclhidden - the encoded name of the hidden field for the close hyperlink name
 * wclMessageClosed - the encodedName of the hidden field for the ids
 */
function msgAct(name, ids, formName, wclhidden, wclMessageClosed) {
   if (document != null && document.forms != null && formName != null) {
      var form = document.forms[formName];
      if (form != null) {
         eval( "form." + wclhidden + ".value = '" + name + "'" );
         eval( "form." + wclMessageClosed + ".value = '" + ids + "'" );
         form.submit();
      }
   }
   return false;
}

/* @auiml: zMA1193 11-18-08 added */
function doSetValue(elem, value)
{
	var dd = document.getElementsByName(elem);
    if (dd[0])
    {
    	dd[0].value = value;
    }
}

/** WPopupMenu **
 * formName - the encoded name of the form
 * actionName - the unique name of the action being performed
 * actionValue - the name of the component performing the action
 * actionNameEnc - the encoded name of the action being performed
 * menuID - the name of the unique id of the popup menu
 * menuCmd - the encoded name of the action being performed
 * wclhidden - the encoded name of the hidden field
 */
function doPop(formName, actionName, actionValue, actionNameEnc, menuID, menuCmd, wclhidden)
{
   if (document != null && document.forms != null && formName != null) {
       var form = document.forms[formName];
       if (form != null) {
           if (actionName != null && actionNameEnc != null && wclhidden != null) {
           	 doSetValue(actionNameEnc,actionValue); //@auiml
             eval("form." + wclhidden + ".value = '" + actionName + "'");
           }
           // We have to store the command name as a hidden field so that it gets passed
           // along when the form is submitted.
           if (menuID != null) {
           	 doSetValue(menuID,menuCmd); //@auiml
           }

           form.submit();
       }
   }
   return false;
}

/** WTable **
 * toggles the background color for a row in the wtable
 */
function doTgl(inputElement) {
   if (inputElement != null)
   {
      // AJAX
      var cells = null;
      //[#108688]-Accessibility - duplicated ids
      if (inputElement.parentNode.parentNode.parentNode.id.substring(0, 14) == "AJAX_CONTAINER") { // @auiml
         cells = inputElement.parentNode.parentNode.parentNode.parentNode.parentNode.childNodes;
      }
      else {
         cells = inputElement.parentNode.parentNode.parentNode.childNodes;
      }
      if (cells != null)
      {
         var suffix = cells[0].className.substring(cells[0].className.indexOf("s") > -1 ? 4 : 3);
         var style = "tbl" + (inputElement.checked ? "s" : "") + suffix;
         for (var i=0; i<cells.length; i++)
         {
            cells[i].className = style;
         }
      }
   }
   return true;
}

/** WTable **
 * toggles radio buttons in wtable
 */
function doRTgl(radioElement) {
   if (radioElement != null) {
      var selected = getSelectedRadio(radioElement);
      if (selected != null) {
         done = doTgl(selected);
         if (selected == radioElement)
            radioElement.checked = false;
      }
      doTgl(radioElement);
   }
   return true;
}

/** WTable **
 * helper method for toggling radio buttons in wtable
 */
function getSelectedRadio(radioElement)
{
  var selected = null;

  if (radioElement != null) {
     var radioGroup = radioElement.form.elements[radioElement.name];

     if (!radioGroup.length)
        radioGroup = new Array(radioElement);
     for (var i=0; selected==null && i<radioGroup.length; i++)
        // AJAX
        if (radioGroup[i].parentNode.parentNode.id && radioGroup[i].parentNode.parentNode.parentNode.parentNode.className.indexOf("tbls") == 0)
           selected = radioGroup[i]
        else if (radioGroup[i].parentNode.parentNode.className.indexOf("tbls") == 0)
           selected = radioGroup[i]
     }

  return selected;
}

/** WTable (number filter)**
 * formName - the encoded name of the form
 * conditionsName - the encoded name of the conditions dropdown
 * startNumberName - the encoded name of the start number
 * endNumberName - the encoded name of the end number
 */
function numUpdate(formName, conditionsName, startNumberName, endNumberName) {
   var form = document.forms[formName];
     if (form != null) {
        var index = eval("form." + conditionsName + ".selectedIndex;");
        eval("form." + startNumberName+ ".parentNode.parentNode.parentNode.style.visibility=index == 0 ? 'hidden' : 'visible'");
        eval("form." + endNumberName + ".parentNode.parentNode.parentNode.style.visibility=index !=7 && index != 8? 'hidden' : 'visible'");
     }
   return true;
}

/** WTable (date filter)**
 * formName - the encoded name of the form
 * conditionsName - the encoded name of the conditions dropdown
 * startDateDivId - id of div containing the start date chooser
 * startTimeDivId - id of div containing the start time chooser
 * endDateDivId - id of div containing the end date chooser
 * endTimeDivId - id of div containing the end time chooser
 */
function dateUpdate(formName, conditionsName, startDateDivId, startTimeDivId, endDateDivId, endTimeDivId) {
   var form = document.forms[formName];
     if (form != null) {
        var index = eval("form." + conditionsName + ".selectedIndex;");
        var startVisiblity = (index != 0) ? 'visible' : 'hidden';
        var endVisibility = (index == 3) ? 'visible' : 'hidden';
        document.getElementById(startDateDivId).style.visibility = startVisiblity;
        document.getElementById(startTimeDivId).style.visibility = startVisiblity;
        document.getElementById(endDateDivId).style.visibility = endVisibility;
        document.getElementById(endTimeDivId).style.visibility = endVisibility;
     }
   return true;
}

/** WTable (string filter)**
 * formName - the encoded name of the form
 * conditionsName - the encoded name of the conditions dropdown
 * textName - the encoded name of the text entry
 * matchcaseName - the encoded name of the matchcase checkbox
 */
function stringUpdate(formName, conditionsName, textName, matchcaseName) {
   var form = document.forms[formName];
     if (form != null) {
        var index = eval("form." + conditionsName + ".selectedIndex;");
        if (index == undefined) {
        	index = eval("form." + conditionsName + "[0].selectedIndex;");
        	eval("form." + textName+ "[0].parentNode.parentNode.parentNode.parentNode.parentNode.style.visibility=index == 5 || index == 6 ? 'hidden' : 'visible'");
       	    eval("form." + matchcaseName+ "[0].parentNode.style.visibility=index == 5 || index == 6 ? 'hidden' : 'visible'");
        } else {
        	eval("form." + textName+ ".parentNode.parentNode.parentNode.parentNode.parentNode.style.visibility=index == 5 || index == 6 ? 'hidden' : 'visible'");
        	eval("form." + matchcaseName+ ".parentNode.style.visibility=index == 5 || index == 6 ? 'hidden' : 'visible'");
        }
     }
   return true;
}


/** WFDALayout **
 * Use this method to create a state object to track the current state of
 * the FDA.  The FDA renderer should instantiate a state object and store
 * it in a var with an encoded name to prevent namespace collisions.
 * Components that use the same FDA should refer to the same var by using
 * encodeName on the FDA id.  The state object should be passed into the FDA
 * toggle method.
 *
 * fdaId - the id of the fda layout
 * defaultId - the id of the default div
 */
function fdaState( fdaId, defaultId ) {
   // the id of the fda layout
   this.fdaId = fdaId;
   // the id of the default div
   this.defaultId = defaultId;
   // the id of the div that is currently shown
   this.currentId = this.defaultId;
}

/** WFDALayout **
 * Use this method to toggle the current text displayed in the FDA.
 *
 * fdaStateId - the name of the fdaState object that keeps track of the current FDA text.
 *    the javascript var will be initialized by the FDA renderer, so other
 *    components should refer to the same var by using encodeName on the FDA id.
 * contentId - the id of the fda content to display, or null to disply the default
 */
function fdaDesc( fdaStateId, contentId ) {
    var fdaState = null ;
    var fdaStateDocument = null ;

    // search for enclosing window/frame containing the fdaState named by fdaStateId ...
    if (fdaStateId!=null) {
        for (
               var fdaStateWindow=window
                   ;
               fdaStateWindow==null
                   ;
               fdaStateWindow=fdaStateWindow.parent
            ) {
            if (fdaStateWindow[ fdaStateId ] != null) {
                fdaState = fdaStateWindow[ fdaStateId ] ;
                fdaStateDocument = fdaStateWindow.document ;
                break;
            }
        }
    }

   if ( fdaState != null ) {
      if ( fdaState.currentId != null ) {
         var hideMe = fdaStateDocument.getElementById( fdaState.currentId );
         if ( hideMe != null ) {
             hideMe.style.display = "none";
             fdaState.currentId = null;
         }
      }

      // appending "Div" here is a bit unorthodox, but calls to this function
      // shouldn't need to know about it
      var nextId = ( contentId != null ) ? contentId + "Div" : fdaState.defaultId;
      if ( nextId != null ) {
         var showMe = fdaStateDocument.getElementById( nextId );
         if ( showMe != null ) {
             showMe.style.display = "inline";
             fdaState.currentId = nextId;
         }
         /*
         // this would set the fda to the default content if the requested
         // content was not found.  not sure if this is the expected behavior?
         else if ( nextId != fdaState.defaultId ) {
            nextId = fdaState.defaultId;
            showMe = fdaStateDocument.getElementById( nextId );
            if ( showMe != null ) {
               showMe.style.display = "inline";
               fdaState.currentId = nextId;
            }
         }
         */
      }
   }
   return true;
}

/** WDualList **
 * dualListId - the id of the dual list component
 * formName - the encoded name of the form
 * leftBoxName - the encoded name of the left/from box
 * rightBoxName - the encoded name of the right/to box
 * addButtonName - the encoded name of the add button
 * removeButtonName - the encoded name of the remove button
 * upButtonName - the encoded name of the up button
 * downButtonName - the encoded name of the down button
 */
function dlState( dualListId, formName, leftBoxName, rightBoxName, addButtonName, addAllButtonName, removeButtonName, removeAllButtonName, upButtonName, downButtonName ) {
   this.dualListId = dualListId;
   this.formName = formName;
   this.leftBoxName = leftBoxName;
   this.rightBoxName = rightBoxName;
   this.addButtonName = addButtonName;
   this.addAllButtonName = addAllButtonName;
   this.removeButtonName = removeButtonName;
   this.removeAllButtonName = removeAllButtonName;
   this.upButtonName = upButtonName;
   this.downButtonName = downButtonName;
}

/** WDualList **
 * dualListState - the state object that keeps track of the current dual list.
 *    the javascript var will be initialized by the dual list renderer
 * isLeftBox - true indicates left box, false indicates right box
 */
function dlSelAll( dualListState, isLeftBox ) {
   if ( dualListState != null ) {
      var listName = (isLeftBox) ? dualListState.leftBoxName : dualListState.rightBoxName;
      var listBox = eval( "document.forms['" + dualListState.formName + "']." + listName );
      if ( listBox != null ) {
         for (i = 0; i < listBox.options.length; i++ ) {
            listBox.options[i].selected = true;
         }
      }
   }
}

/** WDualList **
 * dualListState - the state object that keeps track of the current dual list.
 *    the javascript var will be initialized by the dual list renderer
 * isLeftBox - true indicates left box, false indicates right box
 */
function dlMove( dualListState, isLeftBox ) {
   if ( dualListState != null ) {
      var fromName = (isLeftBox) ? dualListState.leftBoxName : dualListState.rightBoxName;
      var toName = (isLeftBox) ? dualListState.rightBoxName : dualListState.leftBoxName;
      var fromList = eval( "document.forms['" + dualListState.formName + "']." + fromName );
      var toList = eval( "document.forms['" + dualListState.formName + "']." + toName );
      if ( toList != null && fromList != null ) {
         toList.selectedIndex = -1;
         var lastIndex = -1;
         var index = fromList.selectedIndex;
         while (-1 != index) {
            lastIndex = index;
            var selectedOption = fromList.options[index];
            var newOption = new Option( selectedOption.text, selectedOption.value );
            toList.options[toList.options.length] = newOption;
            toList.options[toList.options.length - 1].selected = true;
            fromList.options[index] = null;
            index = fromList.selectedIndex;
         }
         if ((-1 != lastIndex) && (0 < fromList.options.length)) {
            if (lastIndex >= fromList.options.length) {
               lastIndex--;
            }
            fromList.options[lastIndex].selected = true;
         }
      }
   }
}

/** WDualList **
 * dualListState - the state object that keeps track of the current dual list.
 *    the javascript var will be initialized by the dual list renderer
 * isLeftBox - true indicates left box, false indicates right box
 */
function dlMoveAll( dualListState, isLeftBox ) {
	dlSelAll(dualListState, isLeftBox);
	dlMove(dualListState, isLeftBox);
}

/** WDualList **
 * dualListState - the state object that keeps track of the current dual list.
 *    the javascript var will be initialized by the dual list renderer
 * isAddButton - whether the button being used is the add button
 * isLeftBox - true indicates left box, false indicates right box
 */
function dlUpdate( dualListState, isAddButton, isLeftBox ) {
   if ( dualListState != null ) {
      var listName = (isLeftBox) ? dualListState.leftBoxName : dualListState.rightBoxName;
      var listBox = eval( "document.forms['" + dualListState.formName + "']." + listName );
      var buttonName = (isAddButton) ? dualListState.addButtonName : dualListState.removeButtonName;
      var allButtonName = (isAddButton) ? dualListState.addAllButtonName : dualListState.removeAllButtonName;
      var button = eval( "document.forms['" + dualListState.formName + "']." + buttonName );
      var allButton = eval( "document.forms['" + dualListState.formName + "']." + allButtonName );
      if ( button != null && listBox != null ) {
         button.disabled = (-1 == listBox.selectedIndex);
         if ( button.disabled ) {
            button.className = 'b3';
         }
         else {
            button.className = 'b1';
         }
      }
      if ( allButton != null && listBox != null ) {
         allButton.disabled = (listBox.length == 0);
         if ( allButton.disabled ) {
            allButton.className = 'b3';
         }
         else {
            allButton.className = 'b1';
         }
      }
   }
}

/** WDualList **
 * dualListState - the state object that keeps track of the current dual list.
 *    the javascript var will be initialized by the dual list renderer
 * isLeftBox - true indicates left box, false indicates right box
 * direction - negative value indicates moving up, positive value indicates moving down
 */
function dlReorder( dualListState, isLeftBox, direction ) {
   if ( dualListState != null ) {
      var listName = (isLeftBox) ? dualListState.leftBoxName : dualListState.rightBoxName;
      var listBox = eval( "document.forms[ '" + dualListState.formName + "']." + listName );
      if ( listBox != null ) {
         var start = (direction < 0) ? 0 : listBox.options.length - 1;
         var end   = (direction < 0) ? listBox.options.length : -1;
         while ((start != end) &&
                (0 <= start) &&
                (start < listBox.options.length) &&
                listBox.options[start].selected)
         {
            start -= direction;
         }
         for (var i = start; (i != end) && (0 <= i) && (i < listBox.options.length); i -= direction ) {
            if (listBox.options[i].selected) {
               var partner = i + direction;
               if ((0 <= partner) && (listBox.options.length > partner)) {
                  var temp1 = listBox.options[i];
                  var tempText = temp1.text;
                  var tempValue = temp1.value;
                  var tempSelect = temp1.selected;
                  var temp2 = listBox.options[partner];
                  temp1.text = temp2.text;
                  temp1.value = temp2.value;
                  temp1.selected= temp2.selected;
                  temp2.text = tempText;
                  temp2.value = tempValue;
                  temp2.selected = tempSelect;
               }
            }
         }
      }
   }
}


function focusFirstInput(parentId){
	var inputs = document.getElementById(parentId).getElementsByTagName("input");
	var found = false;
	for(var i=0; !found && i<inputs.length; i++){
		var currInput = inputs[i];
		if(currInput.type != 'hidden' && currInput.disabled != true){
			currInput.focus();
			found = true;
		}
	}
}

function formatDojoMenu (menu) {
	var menuChildren =[];
	if(menu.popup==undefined) {
		menuChildren =menu.getChildren();
	} else {
		menuChildren =menu.popup.getChildren();
	}
	for(var i=0;i<menuChildren.length;i++) {
		var menuNode = menuChildren[i];
		if(menuChildren[i].label==null || menuChildren[i].label==""){
		  menuNode.domNode.style.display="none";
		} else {
			var iconDomNode = menuNode.domNode.cells[0];
			if(menuNode.id.substring(0,19) !== "dijit_MenuSeparator")
					iconDomNode.style.display = "none";			
			//Work Around for SubMenu Indicator for TIP 1.1.1.x
			//Changes are in WTable.css under 'Context Menu' for dijitMenuExpand class
			if(menuNode.arrowWrapper !=undefined && menuNode.popup!=undefined){
				var domnode = menuNode.domNode.cells[3];
				domnode.innerHTML="\n <div dojoattachpoint=\"arrowWrapper\" style=\"visibility: visible;\">\n<B>&#187;</B></div>\n ";
				formatDojoMenu(menuNode);
			}
		}
	}
}